#[macro_use] extern crate gfx;
extern crate gfx_window_glutin;
extern crate glutin;
#[macro_use] extern crate log;
extern crate env_logger;

use gfx::{Device, VertexBuffer, RenderTarget};
use gfx::traits::FactoryExt;

pub type ColorFormat = gfx::format::Rgba8;
pub type DepthFormat = gfx::format::DepthStencil;

gfx_defines! {
	vertex Vertex {
		pos: [f32; 2] = "a_Pos",
		color: [f32; 3] = "a_Color",
	}

	pipeline pipe {
		vbuf: gfx::VertexBuffer<Vertex> = (),
		out: RenderTarget<ColorFormat> = "Target0",
	}
}

const TRI_VERTS: [Vertex; 3] = [
	Vertex { pos: [-0.5, -0.5], color: [1.0, 0.0, 0.0] },
	Vertex { pos: [0.5,  -0.5], color: [0.0, 1.0, 0.0] },
	Vertex { pos: [0.0,   0.5], color: [0.0, 0.0, 1.0] },
];

const CLEAR_COLOR: [f32; 4] = [0.1, 0.2, 0.3, 1.0];

fn main() {
	env_logger::init().unwrap();

	let events_loop = glutin::EventsLoop::new();
	let window_builder = glutin::WindowBuilder::new()
		.with_title("Zinnia")
		.with_dimensions(1024, 768)
		.with_vsync();

	let (window, mut device, mut factory, main_color, mut main_depth) =
		gfx_window_glutin::init::<ColorFormat, DepthFormat>(window_builder, &events_loop);

	let mut encoder: gfx::Encoder<_, _> = factory.create_command_buffer().into();
	let (vertex_buffer, slice) = factory.create_vertex_buffer_with_slice(&TRI_VERTS, ());

	let pso = factory.create_pipeline_simple(
		include_bytes!("../shaders/tritest.glslv"),
		include_bytes!("../shaders/tritest.glslf"),
		pipe::new()).unwrap();

	let mut data = pipe::Data {
		vbuf: vertex_buffer,
		out: main_color,
	};

	let mut running = true;
	while running {
		events_loop.poll_events(|glutin::Event::WindowEvent{window_id: _, event}| {
			match event {
				glutin::WindowEvent::KeyboardInput(_, _, Some(glutin::VirtualKeyCode::Escape), _)
					| glutin::WindowEvent::Closed => running = false,

				glutin::WindowEvent::Resized(_w, _h) => {
					gfx_window_glutin::update_views(&window, &mut data.out, &mut main_depth);
				}

				_ => {}
			}
		});

		// draw a frame
		encoder.clear(&data.out, CLEAR_COLOR);
		encoder.draw(&slice, &pso, &data);
		encoder.flush(&mut device);

		// swap
		window.swap_buffers().unwrap();
		// flush
		device.cleanup();
	}
}
